/* ************************************************************************** */
/*                                       _____           _                    */
/*  libp7/internals/byteswap.h          |_   _|__  _   _| |__   ___ _   _     */
/*  | Project: libp7                      | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2017/01/21 12:06:11                               |___/     */
/*                                                                            */
/* ************************************************************************** */
#ifndef LIBP7_INTERNALS_BYTESWAP_H
# define LIBP7_INTERNALS_BYTESWAP_H
# if (defined(_WIN16) || defined(_WIN32) || defined(_WIN64)) \
	&& !defined(__WINDOWS__)
#  define __WINDOWS__
# endif

# include <libp7/config.h>
# include <stdint.h>

# if defined(__linux__) || defined(__CYGWIN__)
#  include <byteswap.h>

# elif defined(__WINDOWS__)
#  include <stdlib.h>
#  define __p7_bswap_16(B) (_byteswap_ushort(B))
#  define __p7_bswap_32(B) (_byteswap_ulong(B))
#  define __p7_bswap_64(B) (_byteswap_uint64(B))

# elif defined(__gint__)
#  define P7_BYTESWAP_INDEPENDANT

# else
#  error "Unsupported platform."
# endif

/*	Independant functions */

# ifdef P7_BYTESWAP_INDEPENDANT
/* define the functions */
static inline unsigned int __p7_bswap_16(unsigned int b)
{
	return (((b & 0xFF00) >> 8) | ((b & 0xFF) << 8));
}

static inline unsigned int __p7_bswap_32(unsigned int b)
{
	b = ((b & 0xFFFF0000) >> 16) | ((b & 0x0000FFFF) << 16);
	b = ((b & 0xFF00FF00) >>  8) | ((b & 0x00FF00FF) <<  8);
	return (b);
}

static inline unsigned int __p7_bswap_64(uint_fast64_t b)
{
	b = ((b & 0xFFFFFFFF00000000) >> 32) | ((b & 0x00000000FFFFFFFF) << 32);
	b = ((b & 0xFFFF0000FFFF0000) >> 16) | ((b & 0x0000FFFF0000FFFF) << 16);
	b = ((b & 0xFF00FF00FF00FF00) >>  8) | ((b & 0x00FF00FF00FF00FF) <<  8);
	return (b);
}

#  define bswap_16(x) __p7_bswap_16(x)
#  define bswap_32(x) __p7_bswap_32(x)
#  define bswap_64(x) __p7_bswap_64(x)
# endif

#endif /* LIBP7_INTERNALS_BYTESWAP_H */
